/*
** Module   : ProgressStatsHelper.java
** Abstract : helper to copy and link to Progress 4GL source code
**
** Copyright (c) 2007-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- -------------------Description--------------------
** 001 GES 20070716   @34526 Helper to copy and link to Progress 4GL
**                           source code.
** 002 GES 20130206          Moved parent class to the pattern package.
*/

package com.goldencode.p2j.uast;

import com.goldencode.p2j.cfg.*;
import com.goldencode.p2j.pattern.*;

/**
 * Helper to copy and link to Progress 4GL source code.
 */
public class ProgressStatsHelper
implements SourceFileHelper
{
   /** Column headings. */
   private static final String[] headings = new String[]
   {
      "Original Source File",
      "Preprocessed Source",
      "Lexer Output",
      "Parser Output",
      "Persisted AST"
   }; 
   
   /**
    * Obtain the list of source file descriptions which is suitable for use
    * as the column headings in the source file index table.
    *
    * @return   Column headings.
    */
   public String[] getHeadings()
   {
      return headings;
   }
   
   /**
    * Provide the list of copy specifications for the given source file and
    * output directory.
    *
    * @param    filename
    *           Original source filename.
    * @param    outDir
    *           Root of the output directory where all copies must be made.
    *
    * @return   The list of source and target copy operations to be made.
    */
   public CopySpec[] getSpecifications(String filename, String outDir)
   {
      // safety first
      if (filename == null)
         return null;
      
      CopySpec[] specs = new CopySpec[5];
      
      String[] sources = new String[5];
      String[] targets = new String[5];
      
      try
      {
         // create the master source filename 
         sources[0] = Configuration.toFile(filename).getAbsolutePath();
      }
      catch (ConfigurationException ce)
      {
         return null;
      }
      
      // now create the rest of the source filenames
      StringBuilder sb = new StringBuilder(sources[0]);
      sb.append(AstGenerator.CACHE_POSTFIX);
      sources[1] = sb.toString();
      
      sb = new StringBuilder(sources[0]);
      sb.append(AstGenerator.LEXER_POSTFIX);
      sources[2] = sb.toString();
      
      sb = new StringBuilder(sources[0]);
      sb.append(AstGenerator.PARSER_POSTFIX);
      sources[3] = sb.toString();
      
      sb = new StringBuilder(sources[0]);
      sb.append(AstGenerator.AST_POSTFIX);
      sources[4] = sb.toString();
      
      // create the target filenames
      sb = new StringBuilder(outDir).append(filename);
      sb.append(".txt");
      targets[0] = sb.toString();
      
      sb = new StringBuilder(outDir).append(filename);
      sb.append(AstGenerator.CACHE_POSTFIX).append(".html");
      targets[1] = sb.toString();
      
      sb = new StringBuilder(outDir).append(filename);
      sb.append(AstGenerator.LEXER_POSTFIX).append(".txt");
      targets[2] = sb.toString();
      
      sb = new StringBuilder(outDir).append(filename);
      sb.append(AstGenerator.PARSER_POSTFIX).append(".txt");
      targets[3] = sb.toString();
      
      sb = new StringBuilder(outDir).append(filename);
      sb.append(AstGenerator.AST_POSTFIX);
      targets[4] = sb.toString();
      
      specs[0] = new CopySpec(sources[0], targets[0]);
      specs[1] = new CopySpec(sources[1],
                              targets[1],
                              CopySpec.Type.TXT_TO_HTML);
      specs[2] = new CopySpec(sources[2], targets[2]);
      specs[3] = new CopySpec(sources[3], targets[3]);
      specs[4] = new CopySpec(sources[4], targets[4]);
      
      return specs;
   }
   
   /**
    * Create a URL to link to the given source file and line number.
    *
    * @param    relPath
    *           Relative path to the root of the output directory where the
    *           source files copies were made.
    * @param    source
    *           Original source filename.
    * @param    line
    *           The line number to which to link.
    *
    * @return   A URL to link to the copied file.
    */
   public String getURL(String relPath, String source, String line)
   {
      StringBuilder sb = new StringBuilder();
      sb.append(relPath)
        .append(source)
        .append(AstGenerator.CACHE_POSTFIX)
        .append(".html#")
        .append(line);
        
      return sb.toString();
   }
}
